/*  File    : integratorCsfcn.c
 *  Abstract:
 *
 *      Example C-file S-function.  
 *
 *      This S-function implements a continuous time integrator.
 *
 *      For more details about S-functions, see simulink/src/sfuntmpl_doc.c.
 * 
 *  Author: Phil Goddard (phil@goddardconsulting.ca)
 *  Date:   Q1, 2007
 */


#define S_FUNCTION_NAME integratorCsfcn
#define S_FUNCTION_LEVEL 2

#include "simstruc.h"

/*====================*
 * S-function methods *
 *====================*/

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, 1);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, 1);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 1)) return;
    ssSetInputPortWidth(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 0, 0);

    if (!ssSetNumOutputPorts(S, 1)) return;
    ssSetOutputPortWidth(S, 0, 1);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);
    ssSetSimStateCompliance(S, USE_DEFAULT_SIM_STATE);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *    Specifiy that we have a continuous sample time.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);        
}

#define MDL_INITIALIZE_CONDITIONS
/* Function: mdlInitializeConditions ========================================
 * Abstract:
 *    Initialize both continuous states to zero.
 */
static void mdlInitializeConditions(SimStruct *S)
{
    real_T *pr = mxGetPr(ssGetSFcnParam(S,0));
    real_T *x0 = ssGetContStates(S);
 
	*x0 = *pr;  /* pr[0]; */
}

#define MDL_CHECK_PARAMETERS
#if defined(MDL_CHECK_PARAMETERS) && defined(MATLAB_MEX_FILE)
  /* Function: mdlCheckParameters =============================================
   * Abstract:
   *    The parameter must be double scalar
   */
  static void mdlCheckParameters(SimStruct *S)
  {
      int_T i;

      for (i = 0; i < ssGetNumSFcnParams(S); i++) {
          real_T *pr;
          int_T   el;
          int_T nEls;
          if (mxIsEmpty(ssGetSFcnParam(S,i)) ||
              !mxIsDouble(ssGetSFcnParam(S,i)) ) {
              ssSetErrorStatus(S,"The parameter must be a finite scalar double");
              return;
          }
          pr = mxGetPr(ssGetSFcnParam(S,i));
          nEls = mxGetNumberOfElements(ssGetSFcnParam(S,i));
          if (nEls != 1){
              ssSetErrorStatus(S,"The paramater must be a scalar double");
              return;
          }
          /* Since there is only one element the following does not */
          /* need to be a loop */
          for (el = 0; el < nEls; el++) {
              if (!mxIsFinite(pr[el]) ||
                  !mxIsNaN(pr[el])) {
                  ssSetErrorStatus(S,"The parameter must be a finite scalar double");
                  return;
              }
          }
      }
  }
#endif /* MDL_CHECK_PARAMETERS */

/* Function: mdlOutputs =======================================================
 * Abstract:
 *      y = x;
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T *y = ssGetOutputPortRealSignal(S,0);
    real_T *x = ssGetContStates(S);

    /* y[0] = x[0] */
    *y = *x;
}

#define MDL_DERIVATIVES
/* Function: mdlDerivatives =================================================
 * Abstract:
 *      xdot = u
 */
static void mdlDerivatives(SimStruct *S)
{
    real_T *dx = ssGetdX(S);
    InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);

    /* xdot = u */
    dx[0] = *uPtrs[0];
}



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    No termination needed, but we are required to have this routine.
 */
static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
